<?php
/**
 *
 * Jambopay payment plugin
 *
 * @author Kibet Enock
 * @package VirtueMart
 * @subpackage payment
 * Copyright (C) 2022 Deveint. All rights reserved.
 * @license http://www.gnu.org/copyleft/gpl.html GNU/GPL, see LICENSE.php
 * VirtueMart is free software. This version may have been modified pursuant
 * to the GNU General Public License, and as distributed it includes or
 * is derivative of works licensed under the GNU General Public License or
 * other free or open source software licenses.
 * See /administrator/components/com_virtuemart/COPYRIGHT.php for copyright notices and details.
 *
 * http://virtuemart.net
 */

defined('_JEXEC') or die('Restricted access');
if (!class_exists('vmPSPlugin')) {
	require(JPATH_VM_PLUGINS . DS . 'vmpsplugin.php');
}



class plgVmPaymentJambopay extends vmPSPlugin {

	function __construct(& $subject, $config) {

		//if (self::$_this)
		//   return self::$_this;
		parent::__construct($subject, $config);
		
		
		$this->_loggable = TRUE;
		$this->tableFields = array_keys($this->getTableSQLFields());
		$this->_tablepkey = 'id';
		$this->_tableId = 'id';
		$varsToPush = array(
			'sandbox' => array(0, 'int'),
			'business_email' => array('', 'char'),
			'password' => array('', 'char'),
			'client_key' => array('', 'char'),
			'phone_number' => array('', 'char'),
			'status_pending' => array('', 'char'),
			'status_success' => array('', 'char'),
			'status_canceled' => array('', 'char'),
		);

		$this->setConfigParameterable($this->_configTableFieldName, $varsToPush);

		vmJsApi::addJScript( '/plugins/vmpayment/jambopay/jambopay/assets/js/admin.js');

		$document = JFactory::getDocument();
		$document->addStyleSheet("https://checkout.jambopay.com/jambopay-styles-checkout.min.css", array('version'=>'auto'));

		unset($this->_script['/virtuemart/media/jui/js/jquery-noconflict.js']);

		
	}

	public function getVmPluginCreateTableSQL() {
		return $this->createTableSQL('Jambopay Table');
	}

	function getTableSQLFields() {

		$SQLfields = array(
			'id' => 'int(11) UNSIGNED NOT null AUTO_INCREMENT',
			'virtuemart_order_id' => 'int(1) UNSIGNED',
			'order_number' => 'char(64)',
			'virtuemart_paymentmethod_id' => 'mediumint(1) UNSIGNED',
			'payment_name' => 'varchar(5000)',
			'payment_order_total' => 'decimal(15,5) NOT null',
			'payment_currency' => 'smallint(1)',
			'email_currency' => 'smallint(1)',
			'cost_per_transaction' => 'decimal(10,2)',
			'cost_percent_total' => 'decimal(10,2)',
			'tax_id' => 'smallint(1)',
			'jambopay_custom' => 'varchar(255) ',
		);
		return $SQLfields;
	}

	/**
	 * @param $product
	 * @param $productDisplay
	 * @return bool
	 */
	function plgVmOnProductDisplayPayment($product, &$productDisplay) {
		return;
		$vendorId = 1;
		if ($this->getPluginMethods($vendorId) === 0) {
			return FALSE;
		}

		return TRUE;
	}

	

	/**
	 * @param $cart
	 * @param $payment_advertise
	 * @return bool|null
	 */
	function plgVmOnCheckoutAdvertise($cart, &$payment_advertise) {

		if ($this->getPluginMethods($cart->vendorId) === 0) {
			return FALSE;
		}
		if (!($selectedMethod = $this->getVmPluginMethod($cart->virtuemart_paymentmethod_id))) {
			return null;
		}
		if (isset($cart->cartPrices['salesPrice']) && $cart->cartPrices['salesPrice'] <= 0.0) {
			return null;
		}

		return;
	}
	


	/**
	 *
	 * @param $cart
	 * @param $order
	 * @return bool|null|void
	 */
	function plgVmConfirmedOrder($cart, $order) {
		
		$document = JFactory::getDocument();
		$document->addScriptDeclaration('
		    document.title = "Make Payment";
		');

		if (!($this->_currentMethod = $this->getVmPluginMethod($order['details']['BT']->virtuemart_paymentmethod_id))) {
			return null; // Another method was selected, do nothing
		}
		if (!$this->selectedThisElement($this->_currentMethod->payment_element)) {
			return FALSE;
		}

		if (!class_exists('VirtueMartModelOrders')) {
			require(VMPATH_ADMIN . DS . 'models' . DS . 'orders.php');
		}
		if (!class_exists('VirtueMartModelCurrency')) {
			require(VMPATH_ADMIN . DS . 'models' . DS . 'currency.php');
		}
		$html='';
		$this->getPaymentCurrency($this->_currentMethod);
		$email_currency = $this->getEmailCurrency($this->_currentMethod);

		$payment_name = $this->renderPluginName($this->_currentMethod, $order);

		$this->debugLog('order number: ' . $order['details']['BT']->order_number, 'plgVmConfirmedOrder', 'message');
		$this->setCart($cart);
		$this->setOrder($order);
		$this->setTotal($order['details']['BT']->order_total);

		$this->_method = $this->getPluginMethod($cart->virtuemart_paymentmethod_id);
		//self::$_this = $this;

        $document = JFactory::getDocument();

		if($this->_method->sandbox==1){
			$url = 'http://196.50.21.51:16220/jambopay-js-checkout.min.js';
			$tknurl = 'http://196.50.21.51:16220/token';
		}else{
			$url = 'https://checkout.jambopay.com/jambopay-js-checkout.min.js';
			$tknurl = 'https://checkout.jambopay.com/Token';
		}
		
		JHtml::_('script', $url, array('version' => 'auto', 'relative' => true), array('async' => 'async'));
		vmJsApi::addJScript( '/plugins/vmpayment/jambopay/jambopay/assets/js/jp.js');
		vmJsApi::addJScript('https://cdnjs.cloudflare.com/ajax/libs/crypto-js/4.0.0/crypto-js.min.js');
		vmJsApi::addJScript('https://cdnjs.cloudflare.com/ajax/libs/twitter-bootstrap/4.5.3/js/bootstrap.bundle.min.js');
		vmJsApi::css('jambopay', 'plugins/vmpayment/jambopay/jambopay/assets/css/');
		
		


		// Prepare data that should be stored in the database
		$dbValues['order_number'] = $order['details']['BT']->order_number;
		$dbValues['payment_name'] = $payment_name;
		$dbValues['virtuemart_paymentmethod_id'] = $cart->virtuemart_paymentmethod_id;
		$dbValues['cost_per_transaction'] = $this->_currentMethod->cost_per_transaction;
		$dbValues['cost_percent_total'] = $this->_currentMethod->cost_percent_total;
		$dbValues['payment_currency'] = $this->_currentMethod->payment_currency;
		$dbValues['email_currency'] = $email_currency;
		$dbValues['payment_order_total'] = $this->getTotal();
		$dbValues['tax_id'] = $this->_currentMethod->tax_id;
		$this->storePSPluginInternalData($dbValues);
		VmConfig::loadJLang('com_virtuemart_orders', TRUE);
		
		$this->_method = $this->getPluginMethod($cart->virtuemart_paymentmethod_id);
		$callback_url 		= JROUTE::_ (JURI::root () . 'index.php?option=com_virtuemart&view=pluginresponse&task=pluginresponsereceived');

		$cst = 1;
		$crl = 0; // Use 2 for a json data stream
      
      	$channel = "VirtueMart";

		
		$biz_email = $this->_method->business_email;
		$password = $this->_method->password;
		$client_key = $this->_method->client_key;
		$phone_numberbb = $this->_method->phone_number;
		

		$curr = shopFunctions::getCurrencyByID($this->_currentMethod->payment_currency, 'currency_code_3');

		
	
		$paymentCurrency = CurrencyDisplay::getInstance($method->payment_currency);
		$totalInPaymentCurrency = round($paymentCurrency->convertCurrencyTo($method->payment_currency, $order['details']['BT']->order_total, false), 2);
		$cd = CurrencyDisplay::getInstance($cart->pricesCurrency);
	
		$this->_virtuemart_paymentmethod_id = $order['details']['BT']->virtuemart_paymentmethod_id;
		$dbValues['payment_name'] = $payment_name;
		$dbValues['order_number'] = $order['details']['BT']->order_number;
		$dbValues['virtuemart_paymentmethod_id'] = $this->_virtuemart_paymentmethod_id;
		$dbValues['payment_currency'] = $curr;
		$dbValues['payment_order_total'] = $totalInPaymentCurrency;
		$this->storePSPluginInternalData($dbValues);
		
		$email = $order['details']['BT']->email;
		$phone = ($order['details']['BT']->phone_1) ? $order['details']['BT']->phone_1 : $order['details']['BT']->phone_2;
		// return $this->processConfirmedOrderPaymentResponse(true, $cart, $order, $code); // return true, empty cart, send order
		$cart->_confirmDone = false;
		$cart->_dataValidated = false;
		$cart->setCartIntoSession();
		// vRequest::setVar('html', $code);
		$html = $this->renderByLayout('hostediframe', array(
			"orderid" => $dbValues['order_number'],
			"cemail" => $email,
			"currency" => $curr,
			"p_number" => $phone,
			"amount" => $dbValues['payment_order_total'],
			"b_email" => $biz_email,
			"callBackUrl" => $callback_url,
			"b_phoneno" => $phone_numberbb,
			"client_key" => $client_key,
			"password" => urlencode($password),
			"url" => $tknurl
			));
		vRequest::setVar('html', $html);
    }


	/**
	 * @param $virtuemart_paymentmethod_id
	 * @param $paymentCurrencyId
	 * @return bool|null
	 */
	function plgVmgetPaymentCurrency($virtuemart_paymentmethod_id, &$paymentCurrencyId) {

		if (!($this->_currentMethod = $this->getVmPluginMethod($virtuemart_paymentmethod_id))) {
			return null; // Another method was selected, do nothing
		}
		if (!$this->selectedThisElement($this->_currentMethod->payment_element)) {
			return FALSE;
		}
		$this->getPaymentCurrency($this->_currentMethod);
		$paymentCurrencyId = $this->_currentMethod->payment_currency;
	}

	/**
	 * @param $html
	 * @return bool|null|string
	 */
	function plgVmOnPaymentResponseReceived(&$html) {

		if (!class_exists('VirtueMartCart')) {
			require(VMPATH_SITE . DS . 'helpers' . DS . 'cart.php');
		}
		if (!class_exists('VirtueMartModelOrders')) {
			require(VMPATH_ADMIN . DS . 'models' . DS . 'orders.php');
		}
		VmConfig::loadJLang('com_virtuemart_orders', TRUE);

		$payment_data = JRequest::get('post');
		
		$app 			= JFactory::getApplication();
		$order_number 		= $payment_data['Order_Id'];

		if (!($virtuemart_order_id = VirtueMartModelOrders::getOrderIdByOrderNumber ($order_number))) {
			return;
		}

		$virtuemart_order_id = VirtueMartModelOrders::getOrderIdByOrderNumber($order_number);

		$this->logInfo('plgVmOnPaymentNotification: virtuemart_order_id  found ' . $virtuemart_order_id, 'message');

		
      
		
		$db = JFactory::getDBO();
		$session = JFactory::getSession();
		$return_context = $session->getId();

		if ($payment_data['Status'] == 'SUCCESS') {
			$new_status = 'C';
			$msg = 'Your transaction was successful.';
			$msg = '<center><div class="alert alert-success" role="alert">
    	<center><strong>'.$msg.'</center></strong></div></center>';
    	$cart = VirtueMartCart::getCart();
		$cart->emptyCart();
		}
		elseif($payment_data['Status'] == 'FAILED'){
			$new_status = 'F';
			$msg = 'Your transaction failed. Please contact the store owner';
			$msg = '<center><div class="alert alert-danger" role="alert">
    	<center><strong>'.$msg.'</center></strong></div></center>';
		}
		else{
			$new_status = 'P';
			$msg = 'Something went wrong. Please contact support';
			$msg = '<center><div class="alert alert-warning" role="alert">
    	<center><strong>'.$msg.'</center></strong></div></center>';
    	}
		
		
		$virtuemart_order_id = VirtueMartModelOrders::getOrderIdByOrderNumber($order_number);
		
		if ($virtuemart_order_id) {
			// send the email only if payment has been accepted
			if (!class_exists('VirtueMartModelOrders'))
			require( JPATH_VM_ADMINISTRATOR . DS . 'models' . DS . 'orders.php' );
			$modelOrder = new VirtueMartModelOrders();
			$orders[$virtuemart_order_id]['order_status'] = $new_status;
			$orders[$virtuemart_order_id]['virtuemart_order_id'] = $virtuemart_order_id;
			$orders[$virtuemart_order_id]['customer_notified'] = 0;
			
			// Take the session directly from the DB
			$modelOrder->updateOrderStatus($orders); 
			
			$order_data = $this->getDataByOrderId($virtuemart_order_id);
			$payment_name = $this->renderPluginName($method);
			$payment_currency = $payment_data['p2'];
			$html = $this->_getPaymentResponseHtml($payment_data, $order_number, $order_data, $payment_currency, $msg);
			// echo $html;
			// remove vmcart
		}
		
		return true;
    }
    
    function emptyCart($session_id=NULL, $order_number=NULL) {

    }

    function _getPaymentResponseHtml($paypal_data, $order_id, $order_data, $payment_currency, $msg) {

		$html = '<table>' . "\n";
		
			$html .= '<thead><th colspan="2"><b>' . $msg ."</b></th></thead>";
			$html .= '<tr><td>Order ID</td><td>' . $order_id ."</td></tr>";
			$html .= '<tr><td>Amount</td><td>' . $payment_currency.'  '.number_format(round($order_data->payment_order_total), 2) ."</td></tr>";

		$html .= '</table>' . "\n";

		return $html;
    }
	



	function displayExtraPluginNameInfo($activeMethod) {
		

		return;

	}

	/**
	 * Display stored payment data for an order
	 *
	 * @see components/com_virtuemart/helpers/vmPSPlugin::plgVmOnShowOrderBEPayment()
	 */
	

	/**
	 * Check if the payment conditions are fulfilled for this payment method
	 * @param VirtueMartCart $cart
	 * @param int $activeMethod
	 * @param array $cart_prices
	 * @return bool
	 */
	protected function checkConditions($cart, $activeMethod, $cart_prices) {

		//Check method publication start
		if ($activeMethod->publishup) {
			$nowDate = JFactory::getDate();
			$publish_up = JFactory::getDate($activeMethod->publishup);
			if ($publish_up->toUnix() > $nowDate->toUnix()) {
				return FALSE;
			}
		}
		if ($activeMethod->publishdown) {
			$nowDate = JFactory::getDate();
			$publish_down = JFactory::getDate($activeMethod->publishdown);
			if ($publish_down->toUnix() <= $nowDate->toUnix()) {
				return FALSE;
			}
		}
		$this->convert_condition_amount($activeMethod);

		$address = $cart->getST();

		$amount = $this->getCartAmount($cart_prices);
		$amount_cond = ($amount >= $activeMethod->min_amount AND $amount <= $activeMethod->max_amount
			OR
			($activeMethod->min_amount <= $amount AND ($activeMethod->max_amount == 0)));

		$countries = array();
		if (!empty($activeMethod->countries)) {
			if (!is_array($activeMethod->countries)) {
				$countries[0] = $activeMethod->countries;
			} else {
				$countries = $activeMethod->countries;
			}
		}
		// probably did not gave his BT:ST address
		if (!is_array($address)) {
			$address = array();
			$address['virtuemart_country_id'] = 0;
		}

		if (!isset($address['virtuemart_country_id'])) {
			$address['virtuemart_country_id'] = 0;
		}
		if (in_array($address['virtuemart_country_id'], $countries) || count($countries) == 0) {
			if ($amount_cond) {
				return TRUE;
			}
		}

		return FALSE;
	}


	public function plgVmOnSelectCheckPayment (VirtueMartCart $cart,  &$msg) {

		return $this->OnSelectCheck ($cart);
	}

	
	public function plgVmDisplayListFEPayment (VirtueMartCart $cart, $selected = 0, &$htmlIn) {

		return $this->displayListFE ($cart, $selected, $htmlIn);
	}


	
	public function plgVmOnSelectedCalculatePricePayment(VirtueMartCart $cart, array &$cart_prices, &$cart_prices_name) {
		if (!($selectedMethod = $this->getVmPluginMethod($cart->virtuemart_paymentmethod_id))) {
			return FALSE;
		}
		//$this->isExpToken($selectedMethod, $cart) ;
		return $this->onSelectedCalculatePrice($cart, $cart_prices, $cart_prices_name);
	}


	


	// Checks how many plugins are available. If only one, the user will not have the choice. Enter edit_xxx page
	// The plugin must check first if it is the correct type
	public function plgVmOnCheckAutomaticSelectedPayment(VirtueMartCart $cart, array $cart_prices = array(), &$paymentCounter) {
		return $this->onCheckAutomaticSelected($cart, $cart_prices, $paymentCounter);
	}

	// This method is fired when showing the order details in the frontend.
	// It displays the method-specific data.
	public function plgVmOnShowOrderFEPayment($virtuemart_order_id, $virtuemart_paymentmethod_id, &$payment_name) {
		$this->onShowOrderFE($virtuemart_order_id, $virtuemart_paymentmethod_id, $payment_name);
	}

	// This method is fired when showing when priting an Order
	// It displays the the payment method-specific data.
	public function plgVmonShowOrderPrintPayment($order_number, $method_id) {
		return $this->onShowOrderPrint($order_number, $method_id);
	}

	public function plgVmDeclarePluginParamsPaymentVM3( &$data) {
		return $this->declarePluginParams('payment', $data);
	}

	public function plgVmSetOnTablePluginParamsPayment($name, $id, &$table) {
		return $this->setOnTablePluginParams($name, $id, $table);
	}
	
	
	public function setCart ($cart) {
		$this->cart = $cart;
		if (!isset($this->cart->cartPrices) or empty($this->cart->cartPrices)) {
			$this->cart->prepareCartData();
		}
	}

	public function setOrder ($order) {
		$this->order = $order;
	}

	public function setTotal ($total) {
		if (!class_exists('CurrencyDisplay')) {
			require(VMPATH_ADMIN . DS  .'helpers'.DS.'currencydisplay.php');
		}
		$this->total = vmPSPlugin::getAmountValueInCurrency($total, $this->_method->payment_currency);

		$cd = CurrencyDisplay::getInstance($this->cart->pricesCurrency);
	}

	public function getTotal () {
		return $this->total;
	}

	public function getOrderPassAndPaymentMethodId($order_number){
		$db = JFactory::getDBO ();
		$q = 'SELECT `order_pass`, `virtuemart_paymentmethod_id` FROM `#__virtuemart_orders` WHERE `order_number` ="'.$order_number.'" ';
		$db = JFactory::getDBO ();
		$db->setQuery ($q);

		return $db->loadObject();
	}
	
	

	
}
